#include <\desmet\include\stdio.h>
#define NULL 0
#define OK 1
#define NOTOK 0
#define ERROR -1

/* Copyright (c) 1985 Martin Nohr and Tom Serface
 * All Rights Reserved
 *
 * Revision	Date		Description
 * --------	---------	--------------------------------------------
 *
 * crc		Calculate the crc for a block of text.
 *
 * There are two kinds of checksums that can be used.  The first is
 * simply the sum of the bytes without paying attention to carry.  This will
 * work most of the time, and is 1 byte per packet faster, but the second
 * type CRC is a lot more reliable.  The CRC is two bytes and is calculated
 * as a polynomial.  The algorithm is described in Andrew S. Tanenbaum's
 * book "Computer Networks" and will detect single and double bit errors
 * all errors with an odd number of bits, all burst errors 16 bits or
 * less, 99.997% of 17 bit burst errors, and 99.998% or 18 bit and longer
 * bursts.
 */

short
crc(s,len)
char *s;
int len;
{
	short i, j, crcout;

	for(j=0,crcout=0; j<len; ++j) {
		crcout=crcout ^ (int)*s++ << 8;
		for(i=0; i<8; ++i) {
   	        if(crcout & 0x8000)
				crcout=crcout << 1 ^ 0x1021;
			else
				crcout=crcout << 1;
		}
	}
	return(crcout & 0xFFFF);
}

short
chksum(s,len)
char *s;
int len;
{
	short i, sum;

/*
 * The regular old Checksum
 */
	for (i=0,sum=0 ; i<len ; ++i)
		sum += *s++;
	return(sum);
}

/*
 * crcfile	Calculate a 2 byte CRC for a file.
 */

short
crcfile(file)
char *file;
{
	FILE fp;
	short crcout=0;
	char c, buf[1024];
	int num, i;

	if((fp=fopen(file,"r")) == NULL)	/* File Not Found */
		return ERROR;
	
	while(num=fread(buf,1024,1,fp))
			crcout=crc(buf,num);
	fclose(fp);
	return(crcout);
}

