/* Copyright (c) 1985 Martin Nohr and Tom Serface
 * All Rights Reserved
 *
 * Revision	Date		Description
 * --------	---------	--------------------------------------------
 *
 * term	- Returns Term for a Loan
 *		This calculates the number of periods (e.g., months)
 *		a loan will	exist given the principal, interest,
 *		number of payments per year, and the payment per period
 *		for the loan.
 *			Call as: term(p,i,n,y);
 *			Where:
 *				p = principal of the loan
 *				i = annual interest rate
 *				n = number of periods per year
 *				y = number of total periods
 *
 * double term();
 * double p,i;
 * int n,y;
 */

int
term(p,i,n,r)	/* Term of a loan */
double p,i,r;
int n;
{
	int periods=0;
	double x, result;

	if(round(p*(i/n),2) < round(r,2)) {
		x=(double)(n);
		result=round((-log(1.0-(p*i/x/r))/log(1.0+(i/x))*(1.0/x)),1);
		periods=((int)(result)*x)+(((result-(int)(result))*12.0)+.5);
		return(periods);
	} else {
		printf("\nYou need a payment of at least %10.2f to cover the\n",
			round(p*(i/n),2));
		printf("interest, and at that you will never repay the loan!\n");
		return(0);
	}
}
