/*
	File: TSR-MAIN.C

	Copyright, 1987

	Permission granted for use in non-commercial 
	applications when credit is given to:

		Dan Lewis
		Key Software Products
		440 Ninth Avenue
		Menlo Park, CA 94025
		(415) 364-9847

	The code below illustrates how to use the assembly language
	support routines found in TSR-HELP.A to write a Terminate and
	Stay Resident program.  These routines are designed to allow
	you to code your routine entirely in C.

	The assembly language support code takes advantage of several
	undocumented features of DOS to allow your C function to do
	whatever it wants (including file I/O) without having to worry
	about DOS re-entrancy problems.  These undocumented DOS features
	include:

		(1) The COMMAND.COM Background Task:  INT 28h is called
		    by COMMAND.COM while it is waiting for input from
		    the keyboard.  During this time, the "In-DOS" flag
		    (explained below) will be non-zero, but DOS calls
		    are permissible.

		(2) The "In-DOS" flag:  A far pointer to this byte
		    is returned in ES:BX by function 34h of INT 21h.
		    The byte is non-zero whenever the processor is
		    executing code within DOS, and implies that you
		    should not try to invoke DOS (re-entrantly) at 
		    the moment (except via INT 28h).

		(3) Get Current PSP: The paragraph address of the
		    current PSP is returned in BX by function 51h
		    of INT 21h.

		(4) Set Current PSP:  You can switch to another PSP
		    by loading BX with the paragraph address of the
		    new PSP and executing function 50h of INT 21h.

	Note that (3) and (4) above are required for DOS to maintain
	certain information in the PSP of hte interrupted program that
	relates to files - more specifically file handles.

	Not included, however, is assembly language code to preserve,
	redirect, and restore the Critical Error (INT 24h) or Ctrl-Break 
	(INT 23h) interrupt handlers, or the Terminate Address (stored
	in interrupt vector 22h).  These should be added for a completely
	robust implementation.
*/

#define	PRIVATE	static
#define	PUBLIC	extern
#define	BOOL	int

PUBLIC	long	Get_Vector() ;
PUBLIC	void	Set_Vector() ;
PUBLIC	void	Simulate_Int() ;
PUBLIC	BOOL	Resident() ;
PUBLIC	void	Tsr() ;

PRIVATE	void	Print_Screen() ;
PRIVATE	long	old_int5_vec ;

PUBLIC	void main()
	{
	/*
	------------------------------------------------------
	Retrieve the existing contents of the interrupt vector
	that will be used to "wake up" this TSR program.  This
	vector can then be used for two purposes:

	(1) To pick up the code segment	address of the current
	    interrupt routine to see if this code has already
	    been installed, and

	(2) To save a far pointer to the old interrupt routine
	    in case this new software ever wants to jump to it.
	------------------------------------------------------
	*/
	old_int5_vec = Get_Vector(5) ;


	/*
	---------------------------------------------------------
	Check for residency.  If this software has already been
	installed, then DON'T install a second copy!  Rather,
	issue a simple message and return.  Note, however, that
	the code segment address of the installed version is
	available as the most significant word of "old_int5_vec",
	and this can be used (with some minor work) to get to
	the installed TSR's data segment to alter operating
	parameters if desired.
	---------------------------------------------------------
	*/
	if (Resident(old_int5_vec))
		{
		puts("\nNew Print Screen already resident...") ;
		exit(0) ;
		}


	/*
	-------------------------------------------------------------
	Inform the assembly language support software which interrupt
	vector should "wake-up" this TSR, and also which C function
	should be executed.
	-------------------------------------------------------------
	*/
	Set_Vector(5, Print_Screen) ;
	Tsr() ;
	}


PRIVATE void Print_Screen( /* No arguments or returned result! */ )
	{
	.
	.
	/* Determine whether to use the old or new print screen code... */
	.
	.
	if (use_old)
		{
		Simulate_Int(old_int5_vec) ;
		}
	else
		{
		.
		.
		/* New print screen code goes here... */
		.
		.
		}
	}
